// Avoid conflict with X11 definitions of True/False
#define GFX_NO_BOOL_MACROS
#include <iostream.h>
#include <gfx/std.h>

#include <gfx/GL/gl.h>
#include <gfx/GL/utils.h>
#include <gfx/manip/Arcball.h>


void Arcball::init(const Bounds& bounds, float aspect)
{
    ball = Ball_Alloc();
    Ball_Init(ball, aspect);

    Ball_UseSet(ball, NoAxes);
    Ball_SetBallType(ball, ArcBall);

    center.x = bounds.center[X];
    center.y = bounds.center[Y];
    center.z = bounds.center[Z];
    center.w = 1;

    radius = bounds.radius;

    // Ball_Place(ball, qOne, 0.75);
    Ball_Place(ball, qOne, 0.9);

    camx = camy = camz = 0.0f;

    dragging = NoDrag;
}

int Arcball::mouse_down(int button, int /*state*/, int x, int y)
{
    mousex = startx = x;
    mousey = starty = y;

    if( button == Button1 )
    {
	int width, height;
	glGetViewport(NULL, NULL, &width, &height);

	vNow.x = 2.0*startx/(float)width-1.0;
	vNow.y = -2.0*starty/(float)height+1.0;

	Ball_Mouse(ball, vNow);

	Ball_BeginDrag(ball);

	dragging = Spin;
    }
    else if( button == Button2 )
    {
	dragging = Dolly;
    }
    else if( button == Button3 )
    {
	dragging = Zoom;
    }

    return 1;
}


int Arcball::mouse_up(int /*button*/, int /*state*/, int /*x*/, int /*y*/)
{
    startx = mousex = 0;
    starty = mousey = 0;

    if( dragging == Spin )
	Ball_EndDrag(ball);

    dragging = NoDrag;

    return 0;
}


int Arcball::motion(int x, int y)
{
    if( !dragging )
	return False;

    if( dragging == Spin )
    {
	mousex = x;
	mousey = y;

	int width, height;
	glGetViewport(NULL, NULL, &width, &height);

	vNow.x = 2.0*mousex/(float)width-1.0;
	vNow.y = -2.0*mousey/(float)height+1.0;

	Ball_Mouse(ball, vNow);
    }
    else if( dragging == Dolly )
    {
        GLint last_mode; glGetIntegerv(GL_MATRIX_MODE, &last_mode);
        glMatrixMode(GL_PROJECTION);

        double to_x, to_y, to_z;
        double from_x, from_y, from_z;

        glUnproject(x,y,0, &to_x, &to_y, &to_z);
        glUnproject(mousex,mousey,0, &from_x, &from_y, &from_z);
 
        // The constants below are just acceleration factors
        // to make things move more quickly.
        // glTranslated(8.0*(to_x-from_x), 8.0*(from_y-to_y), 0);
        camx += 11.6*radius*(to_x-from_x);
        camy += 11.6*radius*(from_y-to_y);

        mousex = x;
        mousey = y;
        glMatrixMode((GLenum)last_mode);
    }
    else if( dragging == Zoom )
    {
        GLint last_mode; glGetIntegerv(GL_MATRIX_MODE, &last_mode);
        glMatrixMode(GL_PROJECTION);
        camz += 0.02*radius*(float)(y - mousey);
        mousey = y;
        glMatrixMode((GLenum)last_mode);
    }

    return 1;
}



void Arcball::apply()
{
    Ball_Update(ball, -1);

    GLint last_mode; glGetIntegerv(GL_MATRIX_MODE, &last_mode);

    Ball_Draw(ball, 0, 0, 0);


    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glTranslatef(camx, camy, camz);


    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();

         // glLoadIdentity();
         // panning translation
         // gluLookAt()??

    HMatrix mNow;
    Ball_Value(ball, mNow);

    glTranslatef(center.x, center.y, center.z);
    glMultMatrixf((float *)mNow);
    glTranslatef(-center.x, -center.y, -center.z);

    glMatrixMode((GLenum)last_mode);
}

void Arcball::unapply()
{
    GLint last_mode; glGetIntegerv(GL_MATRIX_MODE, &last_mode);

    glMatrixMode(GL_PROJECTION);
    glPopMatrix();

    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();

    glMatrixMode((GLenum)last_mode);
}

void Arcball::draw()
{
    Ball_Draw(ball, true, true, true);
}
