// Avoid conflict with X11 definitions of True/False
#define GFX_NO_BOOL_MACROS
#include <iostream.h>
#include <gfx/std.h>

#include <gfx/GL/gl.h>
#include <gfx/GL/utils.h>
#include <gfx/manip/Bogospin.h>


BogoSpin::BogoSpin()
{
    rotx = roty = rotz = 0.0f;
    camx = camy = camz = 0.0f;
    dragging = NoDrag;
}

int BogoSpin::mouse_down(int button, int /*state*/, int x, int y)
{
    statex = x;
    statey = y;

    if( button == Button1 )
        dragging = Spin;
    else if( button == Button2 )
        dragging = Dolly;
    else if( button == Button3 )
        dragging = Zoom;

    return False;
}

int BogoSpin::mouse_up(int /*button*/, int /*state*/, int /*x*/, int /*y*/)
{
    dragging = NoDrag;
    return False;
}

int BogoSpin::motion(int x, int y)
{
    if( !dragging )
        return False;

    if( dragging==Spin ) {
        rotx += (float)(y - statey);
        roty += (float)(x - statex);

        statex = x;
        statey = y;

    } else if( dragging==Dolly ) {
        GLint last_mode; glGetIntegerv(GL_MATRIX_MODE, &last_mode);
        glMatrixMode(GL_PROJECTION);
        /*
        glTranslatef((float)(x - statex), (float)(statey - y), 0.0f);
        */

        double to_x, to_y, to_z;
        double from_x, from_y, from_z;

        glUnproject(x,y,0, &to_x, &to_y, &to_z);
        glUnproject(statex,statey,0, &from_x, &from_y, &from_z);
 
        // The constants below are just acceleration factors
        // to make things move more quickly.
        // glTranslated(8.0*(to_x-from_x), 8.0*(from_y-to_y), 0);
        camx += 8.0*(to_x-from_x);
        camy += 8.0*(from_y-to_y);

        statex = x;
        statey = y;
        glMatrixMode((GLenum)last_mode);

    } else if( dragging==Zoom ) {
        GLint last_mode; glGetIntegerv(GL_MATRIX_MODE, &last_mode);
        glMatrixMode(GL_PROJECTION);
        //glTranslatef(0, 0, 0.1*(float)(y - statey));
        camz += 0.1*(float)(y - statey);
        statey = y;
        glMatrixMode((GLenum)last_mode);
    }

    return True;
}

void BogoSpin::apply()
{
    GLint last_mode; glGetIntegerv(GL_MATRIX_MODE, &last_mode);

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glTranslatef(camx,camy,camz);

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();

    glRotatef(rotx, 1.0, 0.0, 0.0);
    glRotatef(roty, 0.0, 1.0, 0.0);
    glRotatef(rotz, 0.0, 0.0, 1.0);

    glMatrixMode((GLenum)last_mode);
}


void BogoSpin::unapply()
{
    GLint last_mode; glGetIntegerv(GL_MATRIX_MODE, &last_mode);

    glMatrixMode(GL_PROJECTION);
    glPopMatrix();

    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();

    glMatrixMode((GLenum)last_mode);
}
