/************************************************************************

  Ellipsoid drawing stuff.
  $Id: VisQuadrics.cxx,v 1.1 1997/06/25 15:24:58 garland Exp $

 ************************************************************************/

#include <gfx/std.h>
#include <gfx/GL/gl.h>
#include <gfx/math/Mat4.h>
#include <gfx/GL/utils.h>

#include "VisQuadrics.h"

GLUquadricObj *newEllipsoidContext()
{
    GLUquadricObj *quadric;

    quadric = gluNewQuadric();
    gluQuadricNormals(quadric, (GLenum)GLU_SMOOTH);
    gluQuadricDrawStyle(quadric, (GLenum)GLU_FILL);
    gluQuadricOrientation(quadric, (GLenum)GLU_OUTSIDE);

    return quadric;
}

static
void red_lighting()
{
    GLfloat mat_diffuse[]  = { 0.4, 0.1, 0.1, 1.0 };
    GLfloat mat_specular[] = { 0.4, 0.1, 0.1, 1.0 };
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_diffuse);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
}

static
void green_lighting()
{
    GLfloat mat_diffuse[]  = { 0.1, 0.4, 0.1, 1.0 };
    GLfloat mat_specular[] = { 0.1, 0.4, 0.1, 1.0 };
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_diffuse);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
}


void lightEllipsoids(int color)
{
    glEnable(GL_LIGHTING);
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, true);

    GLfloat mat_ambient[]  = { 0.1, 0.1, 0.1, 1.0 };
    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, mat_ambient);
    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 10.0);

    if( color == ELLIPSOID_RED )
	red_lighting();
    else
	green_lighting();
}

void drawEllipsoid(GLUquadricObj *quadric, const Mat4& Q_orig,
		   real radius, Vec3 *v)
{
    Mat4 Q = Q_orig;
    Vec4 d;

    if( cholesky(Q, d) )
    {
        // Set the diagonal
        Q(0,0) = d(0);  Q(1,1) = d(1);  Q(2,2) = d(2);  Q(3,3) = d(3);
        // Nuke the upper triangle
        Q(0,1) = Q(0,2) = Q(0,3) = 0;
        Q(1,2) = Q(1,3) = 0;
        Q(2,3) = 0;

        glPushMatrix();

        Mat4 N;
        Q = Q.transpose();
        Q.inverse(N);

	glMultMatrix(N);
        gluSphere(quadric, radius, 10, 10);
        glPopMatrix();
    }
    else if( v )
    {
        glPushAttrib(GL_LIGHTING_BIT);
	red_lighting();

        glPushMatrix();
        glTranslated((*v)[X], (*v)[Y], (*v)[Z]);
        gluSphere(quadric, 0.0025, 3, 3);
        glPopMatrix();

        glPopAttrib();
    }
}
